__package__ = None
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait as wait
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.action_chains import ActionChains
import selenium.common.exceptions as exception
from sqlalchemy import create_engine

import pandas as pd
import json
import csv
import time

#Customized options for Chrome
options = webdriver.ChromeOptions()
options.add_argument('--headless')
options.add_argument('--no-sandbox')
options.add_argument("--start-maximized")
desired_capabilities = options.to_capabilities()

cars_data1 = []

class Carscraper:
    '''
    This class is used to represent a car scraper.

    Attributes:
        manufacturer: The manufacturer of the car.
        model: The model of the car.
        sale_price: The sale_price of the car.
        year: The year of the car.
        transmission: The transmission of the car.
        fuel: The fuel of the car.
        mileage: The mileage of the car.
        condition: The condition of the car.
        location: The location of the car.
        contact_number: The contact_number of the car.
    '''

    def __init__(self):
        self.manufacturer = []
        self.model = []
        self.sale_price = []
        self.year = []
        self.transmission = []
        self.fuel = []
        self.mileage = []
        self.condition = []
        self.location = []
        self.contact_number = []
        self.new_list = []
        self.car_data = {}
        #self.page = range(1,31)
        self.driver = webdriver.Chrome(r"C:\Users\Simeon\Downloads\chromedriver_win32\chromedriver", desired_capabilities=desired_capabilities)

    def get_links(self):
        '''
        This function is used to get webpages (1-30), number of cars and hrefs for the cars listed
        on each of the ebay webpages.

        '''
        #self.page_urls = []
        for page in range(1, 2):
            ROOT = f"https://www.ebay.co.uk/b/Cars/9801/bn_1839037?page=%7Bpage%7D&_pgn=" + str(page)
            self.driver.get(ROOT)
            time.sleep(3)
            print(ROOT)
            #print(self.page_urls)
            listings = self.driver.find_elements_by_xpath("//ul[@class='b-list__items_nofooter srp-results srp-grid']/li")
            # print(listings)
            print(len(listings))
            cars = self.driver.find_elements_by_xpath("//div[@class='s-item__info clearfix']/a")
            for i in cars:
                time.sleep(1)
                self.new_list.append(i.get_attribute("href"))
                time.sleep(3)
            print(self.new_list)

    def get_car_details(self):
        '''
        This function is used to get specific properties (i.e sale_price, model etc)
        of the cars listed or advertised cars on each of the webpage of used cars 
        on ebay and append them to a dictionary; self.car_data.
        '''
        for car in self.new_list:
            self.driver.get(car)
            time.sleep(2)
            self.car_data = {"manufacturer": self.manufacturer, "model": self.model, "sale_price": self.sale_price, "year": self.year, "transmission": self.transmission, "fuel": self.fuel,
                        "mileage": self.mileage, "condition": self.condition, "location": self.location,
                        "contact_number": self.contact_number}

            try:
                manufacturer = self.driver.find_element_by_xpath(
                    "//td[contains(text(),'Manufacturer:')]/following-sibling::td/span").text
                self.car_data['manufacturer'].append(manufacturer)
            except NoSuchElementException:
                self.car_data['manufacturer'].append(None)
            try:
                model = self.driver.find_element_by_xpath("//td[contains(text(),'Model:')]/following-sibling::td/span").text
                self.car_data['model'].append(model)

            except NoSuchElementException:
                self.car_data['model'].append(None)

            try:
                sale_price = self.driver.find_element_by_xpath('//*[@id="prcIsum"]').text
                self.car_data['sale_price'].append(sale_price)

            except NoSuchElementException:
                self.car_data['sale_price'].append(None)

            try:
                year = self.driver.find_element_by_xpath("//td[contains(text(),'Year:')]/following-sibling::td/span").text
                self.car_data['year'].append(year)

            except NoSuchElementException:
                self.car_data['year'].append(None)

            try:
                transmission = self.driver.find_element_by_xpath(
                    "//td[contains(text(),'Transmission:')]/following-sibling::td/span").text
                self.car_data['transmission'].append(transmission)

            except NoSuchElementException:
                self.car_data['transmission'].append(None)

            try:
                fuel = self.driver.find_element_by_xpath("//td[contains(text(),'Fuel:')]/following-sibling::td/span").text
                self.car_data['fuel'].append(fuel)

            except NoSuchElementException:
                self.car_data['fuel'].append(None)

            try:
                mileage = self.driver.find_element_by_xpath("//td[contains(text(),'Mileage:')]/following-sibling::td/span").text
                self.car_data['mileage'].append(mileage)

            except NoSuchElementException:
                self.car_data['mileage'].append(None)

            try:
                condition = self.driver.find_element_by_xpath('//div[@class="u-flL condText  "]').text
                self.car_data['condition'].append(condition)

            except NoSuchElementException:
                self.car_data['condition'].append(None)

            try:
                location = self.driver.find_element_by_xpath('//span[@itemprop="availableAtOrFrom"]').text
                self.car_data['location'].append(location)

            except NoSuchElementException:
                self.car_data['location'].append(None)

            try:
                contact_number = self.driver.find_element_by_xpath('//div[@id="slrCntctNum"]').text
                self.car_data['contact_number'].append(contact_number)

            except NoSuchElementException:
                self.car_data['contact_number'].append(None)

        time.sleep(2)
        self.driver.back()
        time.sleep(2)
        print(self.car_data)
        #cars_data1.append(car_data)


    def print_to_json_and_csv(self):
        '''
        This function is used to print the self.car_data dictionary to a
        json file and subsequently to a dataframe & csv file.
        '''
        with open("car_data.json", "w") as g:
            json.dump(self.car_data, g)
        with open(r"C:\Users\Simeon\PycharmProjects\ebay_car_scraper_pypi\car_scraper\car_data.json", 'r') as f:
            self.car_data = json.load(f)
            self.car_data_df = pd.DataFrame(self.car_data, columns = ['manufacturer', 'model', 'sale_price', 'year', 'transmission', 'fuel', 'mileage', 'condition', 'location', 'contact_number'])
            #print(self.car_data_df)
            self.car_data_df.to_csv("car_data_df.csv")
        return self.car_data_df

    def run(self):
        '''
        This function is used to run or execute all the methods.
        '''
        self.get_links()
        #self.get_cars()
        self.get_car_details()
        self.print_to_json_and_csv()

scraper = Carscraper()
scraper.run()


import sys

import pandas as pd
import psycopg2
from psycopg2 import errors
from sqlalchemy import create_engine
DATABASE_TYPE = 'postgresql'
DBAPI = 'psycopg2'
HOST = 'aicoredb.cjpo05djrpn0.eu-west-2.rds.amazonaws.com'
USER = 'postgres'
PASSWORD = 'Orebach94*'
PORT = 5432
DATABASE = 'postgres'

#conn = psycopg2.connect("dbname=dbname user=user")
conn = psycopg2.connect(dbname=DATABASE, user=USER, password=PASSWORD, port=PORT, host=HOST)
cur = conn.cursor()
#pg = '''DROP table IF EXISTS cars '''
#pg = "DROP table IF EXISTS cars"
pg = "DROP TABLE IF EXISTS cars"
print(pg)
cur.execute(pg)
conn.commit()
cur.execute("CREATE TABLE cars (manufacturer VARCHAR(255), model VARCHAR(255), sale_price VARCHAR(255), year VARCHAR(255), transmission VARCHAR(255), fuel VARCHAR(255), mileage VARCHAR(255), condition VARCHAR(255), location VARCHAR(255), contact_number VARCHAR(255))")
conn.commit()
# except psycopg2.errors.DuplicateTable:
#     print('exists')
#
# else:
#     try:
#      cur.execute(" TABLE cars (manufacturer VARCHAR(255), model VARCHAR(255), sale_price VARCHAR(255), year VARCHAR(255), transmission VARCHAR(255), fuel VARCHAR(255), mileage VARCHAR(255), condition VARCHAR(255), location VARCHAR(255), contact_number VARCHAR(255))")
#     except:
#         pass
# if
# except psycopg2.errors.InFailedSqlTransaction:
#
#     print('Exists')
#
# else:
#     pass


#cars_data_df.to_sql('cars', engine, if_exists='replace', index=False)

#conn = psycopg2.connect("host=localhost dbname=postgres user=postgres")
with open(r"C:\Users\Simeon\PycharmProjects\pythonProject2\AiCore Substitute lecture videos & revision py_files\car_scraper package project 2\car_scraper\car_data_df.csv", 'r') as f:
    cars_data_df = pd.read_csv(f, index_col=0)
    engine = create_engine(f"{DATABASE_TYPE}+{DBAPI}://{USER}:{PASSWORD}@{HOST}:{PORT}/{DATABASE}")
    engine.connect()
    cars_data_df.to_sql('cars', engine, if_exists='replace', index=False)

cur.execute("SELECT * FROM cars")
result = cur.fetchall()
for r in result:
    print(r)

conn.close()
with open(r"C:\Users\Simeon\PycharmProjects\ebay_car_scraper_pypi\README.md", r) as a:
    long_description = a.read()

#     cars_data_df.to_sql('cars', engine, if_exists='replace', index=False)

#     next(f) # Skip the header row.
#     cur.copy_from(f, 'cars', sep=',')
#     conn.commit()

# with cur.cursor() as cur:
#     # Print all the tables from the database
# cur.execute("SELECT * FROM cars")
# result = cur.fetchall()
# for r in result:
#     print(r)


from setuptools import setup
from setuptools import find_packages

# read_me = None
#
# with open(r"C:\Users\Simeon\PycharmProjects\ebay_car_scraper_pypi\README.md", 'r') as a:
#     read_me = a.read(),

setup(
    name='ee_ff_gg', ## This will be the name your package will be published with
    version='0.0.1',
    description='Mock package that allows extract second-hand car data from ebay',
    #long_description=read_me, #README file
    url='https://github.com/Simeon94/ebay_car_scraper_pypi.git', # Add the URL of your github repo if published
                                                              # in GitHub
    author='Simeon Bamgbaye', # Your name
    license='MIT',
    packages=find_packages(), # This one is important to explain. See the notebook for a detailed explanation
    install_requires=['selenium'], # For this project we are using two external libraries
                                                     # Make sure to include all external libraries in this argument
)


import sys
import os.path
sys.path.append(os.path.join(os.path.dirname(__file__), r'C:\Users\Simeon\PycharmProjects\car_scraper_packages'))
from car_scraper.car_scraping_1 import Carscraper
import boto3
import psycopg2
import os
import pandas as pd
from sqlalchemy import create_engine


class DataHandling:
    # s3_client = boto3.client('s3')
    # DATABASE_TYPE = 'postgresql'
    # DBAPI = 'psycopg2'
    # HOST = 'aicoredb.cjpo05djrpn0.eu-west-2.rds.amazonaws.com'
    # USER = 'postgres'
    # PASSWORD = 'Orebach94*'
    # PORT = 5432
    # DATABASE = 'postgres'

    def __init__(self):
        self.carscraper = Carscraper()
        self.DATABASE_TYPE = 'postgresql'
        self.DBAPI = 'psycopg2'
        self.HOST = 'aicoredb.cjpo05djrpn0.eu-west-2.rds.amazonaws.com'
        self.USER = 'postgres'
        self.PASSWORD = 'Orebach94*'
        self.PORT = 5432
        self.DATABASE = 'postgres'
        #self.cur = None
        self.df = None
        self.conn = psycopg2.connect(dbname=self.DATABASE, user=self.USER, password=self.PASSWORD, port=self.PORT, host=self.HOST)
        self.cur = self.conn.cursor()

    def load_csv(self):
        #with open(r"C:\Users\Simeon\PycharmProjects\pythonProject2\AiCore Substitute lecture videos & revision py_files\car_scraper package project 2\car_scraper\car_data_df.csv", 'r') as f:
            #self.cars_data_df = pd.read_csv(f, index_col=0)
        self.df = self.carscraper.print_to_json_and_csv()
        #self.df = pd.read_csv(df, index_col=0)
        #print(self.df)

    def create_rds_table(self):
        self.cur.execute("DROP TABLE IF EXISTS cars")
        self.cur.execute("CREATE TABLE cars (manufacturer VARCHAR(255), model VARCHAR(255), sale_price VARCHAR(255), year VARCHAR(255), transmission VARCHAR(255), fuel VARCHAR(255), mileage VARCHAR(255), condition VARCHAR(255), location VARCHAR(255), contact_number VARCHAR(255))")
        self.conn.commit()


    def store_data_in_database(self):
        engine = create_engine(f"{self.DATABASE_TYPE}+{self.DBAPI}://{self.USER}:{self.PASSWORD}@{self.HOST}:{self.PORT}/{self.DATABASE}")
        engine.connect()
        self.df.to_sql('cars', engine, if_exists='replace', index=False)

    def query_database(self):
        self.cur.execute("SELECT * FROM cars")
        result = self.cur.fetchall()
        for r in result:
            print(r)
        self.conn.close()

    def run_data(self):
        self.load_csv()
        self.create_rds_table()
        self.store_data_in_database()
        self.query_database()

pushdata = DataHandling()

pushdata.run_data()

#######################################################################################################
#Adv method:
# def __init__(self, user, password, database, port, host):
#     self.user = user
#     self.password = password
#     self.database = database
#     self.port = port
#     self.host = host
#     self.carscraper = Carscraper()
#
#     global conn
# conn = psycopg2.connect(dbname=self.database, user=self.user, password=self.password, port=self.port, host=self.host)

# def __repr__(self):
#     return "User {0} connected to {1}.".format(self.user, self.database)

# @classmethod
# def get_credentials_and_login(cls, file_path):
#     with open(file_path, mode="r") as json_file:
#         # Store credentials to the global namespace
#         global credentials
#         credentials = json.load(json_file)
#
#     # login
#     return cls(credentials["user"],
#                credentials["password"],
#                credentials["database"],
#                credentials["host"])


# __package__ = None
from selenium import webdriver
import sys
import os.path
sys.path.append(os.path.join(os.path.dirname(__file__), r'C:\Users\Simeon\PycharmProjects\ebay_car_scraper_pypi'))
import car_scraper
import unittest
from car_scraper.car_scraping_1 import Carscraper
import random

#Customized options for Chrome
options = webdriver.ChromeOptions()
#options.add_argument('--headless')
options.add_argument('--no-sandbox')
options.add_argument("--start-maximized")
desired_capabilities = options.to_capabilities()

class CarscraperTestCase(unittest.TestCase):

    def setUp(self):
        self.carscraper = Carscraper()
        self.carscraper.driver = webdriver.Chrome(r"C:\Users\Simeon\Downloads\chromedriver_win32\chromedriver",
                                       desired_capabilities=desired_capabilities)

    # def test_wrong_link(self):
    #     page = 50
    #     with self.assertRaises(ValueError) as err:
    #         self.car_scraping_1.get_links(page)

    def test_no_of_cars_per_page(self):
        # code to test scraper returns the expected number of cars on the webpage
        i = 13
        url = f"https://www.ebay.co.uk/b/Cars/9801/bn_1839037?page=%7Bpage%7D&_pgn=" + str(i)
        self.carscraper.driver.get(url)
        no_of_cars = 48
        xpath = "//ul[@class='b-list__items_nofooter srp-results srp-grid']/li"
        self.carscraper.driver.find_elements_by_xpath(xpath)
        actual_value = len(self.carscraper.driver.find_elements_by_xpath(xpath))
        expected_value = no_of_cars
        self.assertEqual(actual_value, expected_value)


    # def test_correct_url(self):
    #     #code to test scraper return the correct url
    #     url = f"https://www.ebay.co.uk/b/Cars/9801/bn_1839037?page=%7Bpage%7D&_pgn=50"
    #     self.carscraper.driver.get(url)
    #     actual_value = self.carscraper.driver.current_url
    #     expected_value = url
    #     self.assertEqual(actual_value, expected_value)


    #
    #
    # def check_exists(self)
    #
    # def test_wrong_date(self):
    #     date = 'February_30'
    #     with self.assertRaises(ValueError) as err:
    #         self.scraper.get_celebrities(date)
unittest.main(argv=[''], verbosity=0, exit=False)
